// Grafo.cpp: implementation of the Grafo class.
//
//////////////////////////////////////////////////////////////////////



#include "Grafo.h"

char *EdgeTypeS[]={"Tree","Back" ,"Cross" ,"Forward"};

extern HWND winGraphHandle;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

/*************************** Metodi della classe Vertice ***************************************/
Vertice::Vertice(POINTS pos ,COLORREF col) 
{
	Vertice::colore=col;	// attributi grafici
	Vertice::pos=pos;		// attributi grafici
	Vertice::nome=NULL;	
	Vertice::idx=0;		
	Dtime=Ftime=NIL;			// attributi di visita
	color=Bianco;
	predecessore = NIL;
	spessore = SPESSORE_VERTICE;
	d=UNUSED_WEIGHT;

}		

/*************************************************************************************/

Vertice &Vertice::operator=(Vertice &rvalue)
{
	if(&rvalue != this)			// si accerta che non si tratti di un self assignement
	{		
		colore	= rvalue.colore;	// attributi grafici
		pos		= rvalue.pos;		// attributi grafici
		
		idx		= rvalue.idx;		
		Dtime = rvalue.Dtime;		// attributi di visita
		Ftime = rvalue.Ftime;	
		color = rvalue.color;
		predecessore = rvalue.predecessore;
		delete [] nome;
		nome = new char[strlen(rvalue.nome)+1];
		strcpy(nome,rvalue.nome);
	}

return *this;
}

/*************************************************************************************/

bool Vertice::paint(HDC hdc)
{

	HPEN penna=CreatePen( PS_DASH, spessore, colore ) ;
	SelectObject(hdc,penna);
	RECT rect;
	static COLORREF rgb[30]={
							RGB(255,255,255),	// Bianco
							RGB(196,196,196),	// Grigio
							RGB(0,0,0),			// Nero
							RGB(255,0,0),		// Rosso
							RGB(0,255,0),		// Verde
							RGB(0,0,255),		// Blu
							RGB(0,255,255),		// Azzurro
							RGB(255,255,0),		// Giallo
							RGB(255,0,255),		// Fucsia
							RGB(255,128,0),		// Arancione
							RGB(0,128,66),		// Verde scuro
							RGB(255,255,255),
							RGB(255,255,255),
							RGB(255,255,255),
							RGB(255,255,255),
							RGB(255,255,255),
							RGB(255,255,255),
							};
	
	HBRUSH brush = CreateSolidBrush(rgb[color]); 
	
	HGDIOBJ temp = SelectObject(hdc,brush);	

	Ellipse( hdc,pos.x-RAGGIO,pos.y-RAGGIO,pos.x+RAGGIO,pos.y+RAGGIO); 
	rect.left	= pos.x-16;
	rect.top	= pos.y-16;
	rect.right	= pos.x;
	rect.bottom	= pos.y;
	DrawText(hdc, nome, -1, &rect,DT_EXPANDTABS|DT_WORDBREAK);	
	//	GrayString(hdc, brush,NULL,&LL,2,rect.left,rect.top,16,16);
	
	
	char c[6];
	
	if(d != UNUSED_WEIGHT )
	{
	sprintf(c,"%d\0",d);	
	
	rect.left	= pos.x-18;
		rect.top	= pos.y;
		rect.right	= pos.x+18;
		rect.bottom	= pos.y+14;
		DrawText(hdc, c, -1, &rect,DT_EXPANDTABS|DT_WORDBREAK);	
	
	}

	if(Dtime != -1 )
	{
		if(Ftime != -1 )
			sprintf(c,"%d/%d",Dtime,Ftime);
		else
			sprintf(c,"%d/",Dtime);
		
	
		c[5]='\0';

		rect.left	= pos.x-18;
		rect.top	= pos.y;
		rect.right	= pos.x+18;
		rect.bottom	= pos.y+14;
		DrawText(hdc, c, -1, &rect,DT_EXPANDTABS|DT_WORDBREAK);	
	}
	

	SelectObject(hdc,temp);

	DeleteObject(brush);
	DeleteObject(penna);

return true;
}


/*************************** Metodi della classe Arco ***************************************/


bool Arco::paintDirect(HDC hdc)
{
	HPEN	penna	= CreatePen( PS_DASH, spessore, colore ) ; 
	HGDIOBJ temp	= SelectObject(hdc,penna);
	RECT rect;
	POINT lppt[4],PosPeso;
	const double pi=2*asin(1);
	double TETA;
	if(getGrafo()->is_adjacent(to->idx,from->idx)) // occorre tracciarlo curvo ?
		TETA=pi/12;		
	else
		TETA=0;			// in tal caso sarebbe inutile tracciare un arco 
				
	int Ax = from->pos.x;				// calcolo i punti di controllo di Bezier
	int	Ay = from->pos.y;	// ruotando di 30 gradi il vettore AB
	int Bx = to->pos.x;
	int By = to->pos.y ;
	double BAx = Bx-Ax;
	double BAy = By-Ay;

	int Rotx =(int) ((BAx*cos(TETA)-BAy*sin(TETA))*0.5); 
	int Roty =(int) ((BAy*cos(TETA)+BAx*sin(TETA))*0.5);

	lppt[0].x = Ax;
	lppt[0].y = Ay;

	lppt[3].x = Bx;
	lppt[3].y = By;

	lppt[1].x =  Ax+Rotx;
	lppt[1].y =  Ay+Roty;

	lppt[2] = lppt[1] ;  // per il momento non calcolo il secondo punto di controllo						
							
	PolyBezier( hdc, lppt, 4 ); 							
	
	
	TETA = TETA+pi/60;
	Rotx =(int) ((BAx*cos(TETA)-BAy*sin(TETA))*0.5); 
	Roty =(int) ((BAy*cos(TETA)+BAx*sin(TETA))*0.5);
	PosPeso.x =  Ax+Rotx;
	PosPeso.y =  Ay+Roty;

	char PesoArcoStr[9];

	rect.left	= PosPeso.x-15;
	rect.top	= PosPeso.y-10;
	rect.right	= PosPeso.x+15;
	rect.bottom	= PosPeso.y+10;


	TETA = pi/6;	// occorre disegnare la freccetta
							//determino il vertice
							
	double dx = lppt[1].x-Bx;  // la punta della freccia va fatta nella
	double dy = lppt[1].y-By;  // direzione del control point di bezier
	double dist  = sqrt(pow(dx,2)+pow(dy,2));
							
	Ax = (int) (Bx+(dx/dist)*1.5*RAGGIO); 
	Ay = (int) (By+(dy/dist)*1.5*RAGGIO);

	Bx= (int) (Bx+(dx/dist)*RAGGIO);		// trovo la punta della freccia
	By= (int) (By+(dy/dist)*RAGGIO);
	
	BAx = Ax-Bx;
	BAy = Ay-By;
							
	Rotx =(int) (BAx*cos(TETA)-BAy*sin(TETA)); // trovo il vettore di rotazione
	Roty =(int) (BAy*cos(TETA)+BAx*sin(TETA));
						
	MoveToEx(hdc,Bx,By,NULL);
	LineTo(hdc,Bx+Rotx,By+Roty);
							
	TETA=(-TETA);
	Rotx =(int) (BAx*cos(TETA)-BAy*sin(TETA)); 
	Roty =(int) (BAy*cos(TETA)+BAx*sin(TETA));
							
	MoveToEx(hdc,Bx,By,NULL);
	LineTo(hdc,Bx+Rotx,By+Roty);
							
	
			 
   if(getGrafo()->is_Pesato() && peso != UNUSED_WEIGHT)  // se il grafo  pesato etichetto l'arco
	{	
		sprintf(PesoArcoStr,"%d\0",peso);
		DrawText(hdc, PesoArcoStr, -1, &rect,DT_EXPANDTABS|DT_WORDBREAK|DT_CENTER| DT_VCENTER );					
	}
	
	SelectObject(hdc,temp);
	DeleteObject(penna);
	

	return true;
}

/*************************************************************************************/

bool Arco::paintUndirect(HDC hdc)
{
	HPEN	penna = CreatePen( PS_DASH, spessore, colore ) ; 
	HGDIOBJ temp  = SelectObject(hdc,penna);
	//RECT rect;
	
	MoveToEx(hdc,from->pos.x,from->pos.y,NULL);
	LineTo(hdc,to->pos.x,to->pos.y);					
						
	SelectObject(hdc,temp);
	DeleteObject(penna);
	
	return true;

}

/*************************** Metodi della classe Grafo ***************************************/

Grafo::Grafo(TipoDiGrafo tipo , char *label)
{
	TipoGrafo		= tipo;
	NumeroDiArchi	= 0;
	NumeroDiVertici	= 0;
	strncpy(nome,label,MAX_PRINT_BUFFER);	// al piu' MAX_PRINT_BUFFER
	nome[MAX_PRINT_BUFFER]='\0';			// in caso di troncamento
	for(int i=0;i< MAX_VERTICI;i++)	
			for(int j=0;j< MAX_VERTICI;j++)	
				if(M_ADJ[i][j]=NULL);
		
}
/*************************************************************************************/	
Grafo::Grafo(Grafo& unGrafo)				// nel caso Grafo x = y evita memberwise init
{

}
/*************************************************************************************/
Grafo::~Grafo()
{

}

/*************************************************************************************/
Grafo	&Grafo::operator=(Grafo& rvalue) //overload di assegnamento memberwise copy
{

	if(&rvalue != this)			// si accerta che non si tratti di un self assignement
	{
	
	//delete [] vertice;
	//delete [] M_ADJ;
	strncpy (nome,rvalue.nome,MAX_PRINT_BUFFER+1);
	NumeroDiArchi	= rvalue.NumeroDiArchi;
	NumeroDiVertici	= rvalue.NumeroDiVertici;
	time			= rvalue.time;
	TipoGrafo		= rvalue.TipoGrafo;
	

	}

return *this;
}

/*************************************************************************************/

Arco *Grafo::is_adjacent(int V1, int V2)
{
	if ((V1 < NumeroDiVertici) && (V2 < NumeroDiVertici))
		if(M_ADJ[V1][V2] != NULL)	
			return M_ADJ[V1][V2];
		
	return NULL;
}

/*************************************************************************************/

bool	Grafo::add_vertex(Vertice* v)				/* Aggiunge un Vertice*/
{
	if (NumeroDiVertici >= MAX_VERTICI) 
			return false;
	
	vertice[NumeroDiVertici] = v;
	v->nome=strdup("A");
	v->nome[0]='A'+NumeroDiVertici;
	v->idx=NumeroDiVertici;
	NumeroDiVertici++;

		return true;

}

/*************************************************************************************/

bool	Grafo::add_edge(Arco &Edge)			/* Aggiunge un arco da Vertice1 a Vertice2 di peso weight nel grafo G. Se Vertice1 o Vertice2  non sono	vertici del grafo e se weight non e' compreso fra 0 e UNUSEDWEIGHT ritorna ERROR*/
{
return true;
}

/*************************************************************************************/

bool	Grafo::add_edge(Vertice &V1,Vertice &V2)	/* Aggiunge un arco da Vertice1 a Vertice2 di peso weight nel grafo G. Se Vertice1 o Vertice2  non sono	vertici del grafo e se weight non e' compreso fra 0 e UNUSEDWEIGHT ritorna ERROR*/
{
return true;
}

/*************************************************************************************/

bool	Grafo::add_edge(int V1,int V2, int peso )	/* Aggiunge un arco da Vertice1 a Vertice2 di peso weight nel grafo G. Se Vertice1 o Vertice2  non sono	vertici del grafo e se weight non e' compreso fra 0 e UNUSEDWEIGHT ritorna ERROR*/
{
if(V1!=V2)					// niente cappi 
if(M_ADJ[V1][V2]==NULL)		//	se non esiste gia 
{
	NumeroDiArchi++;
	M_ADJ[V1][V2]=new Arco(vertice[V1],vertice[V2],peso,this);
	if (is_Non_Orientato())
		M_ADJ[V2][V1]=M_ADJ[V1][V2];
	return true;
}
	
	return false;
}

/*************************************************************************************/

bool	Grafo::delete_edge(Vertice &V1,Vertice &V2)	/*	Cancella l'arco da Vertice1 a Vertice2 in G. Se Vertice1 o Vertice2 non sono vertici del grafo, o se non c'e' un arco fra loro ritorna ERROR. */
{
return true;
}

/*************************************************************************************/

bool	Grafo::delete_edge(int V1, int V2) 			/*	Cancella l'arco da Vertice1 a Vertice2 in G. Se Vertice1 o Vertice2 non sono vertici del grafo, o se non c'e' un arco fra loro ritorna ERROR. */
{
return true;
}

/*************************************************************************************/

bool	Grafo::delete_edge(Arco &Edge) 			/*	Cancella l'arco da Vertice1 a Vertice2 in G. Se Vertice1 o Vertice2 non sono vertici del grafo, o se non c'e' un arco fra loro ritorna ERROR. */
{
return true;
}

/*************************************************************************************/

bool Grafo::paint(HDC hdc)
{
	if (getTipo()&IDM_ORIENTATO)
		{
		for(int i=0;i<NumeroDiVertici;i++)
			for(int j=0;j<NumeroDiVertici;j++)
				if(M_ADJ[i][j] != NULL)
					M_ADJ[i][j]->paintDirect(hdc);
		
		}
	else		// se il grafo non  orientato
	for(int i=0;i<NumeroDiVertici;i++)
		for(int j=0;j<NumeroDiVertici;j++)
			if(M_ADJ[i][j] != NULL)
				M_ADJ[i][j]->paintUndirect(hdc);
			
	for(int i=0;i<NumeroDiVertici;i++)
		vertice[i]->paint(hdc);
		
		
	return true;
}

/*************************************************************************************/

bool	Grafo::setTipo(int T)
{
	//		occorre reinizalizzare il grafo
	switch (T)
	{
	case IDM_ORIENTATO: 
	case IDM_PESATO:
			TipoGrafo= (TipoDiGrafo)(TipoGrafo  | T); 
			return 	true;   
			
	case IDM_NON_ORIENTATO: 
	case IDM_NON_PESATO:
			TipoGrafo= (TipoDiGrafo)(TipoGrafo & T); 

			return 	true;   
	default:	return 	false;   



	}


}

/*************************************************************************************/

int Grafo::SnapToVertex(POINTS punto,int raggio)   // controlla se il punto  a distanza raggio da un vertice
{
	int i;
	for( i=0; i< NumeroDiVertici;i++)
	if(  sqrt(pow((vertice[i]->pos.x-punto.x),2) + pow((vertice[i]->pos.y - punto.y),2)) < raggio )
		return i;
	return (-1);
}

/*************************************************************************************/

bool Grafo::SnapToGrid(int Grid)   // Auto Format Graph
{
	int i;
	for( i=0; i< NumeroDiVertici;i++)		
		;//vertice[i]->SetPos(pos);
return true;
}

/*************************************************************************************/


void Grafo::DFS(HANDLE hInstanceSockman) 
{

for(int v=0;v< NumeroDiVertici;v++)
{
	vertice[v]->color = Bianco;
	vertice[v]->predecessore  = -1;
}
time=0;

for(v=0;v< NumeroDiVertici;v++)
	if(vertice[v]->color == Bianco)
		DFS_Visit(v, hInstanceSockman);
	
}


#define SNAPSHOT MessageBeep(0);paint(GetDC(h) );Sleep(2000);


void Grafo::DFS_Visit(int u, HANDLE h)
{
vertice[u]->color = Grigio;
vertice[u]->Dtime = time++; 

for(int v=0;v< NumeroDiVertici;v++)
	if(is_adjacent(u,v) != NULL)
	{		
		if(vertice[v]->color == Bianco)
		{
			M_ADJ[u][v]->setTree();
			vertice[v]->predecessore = u;			
			//SNAPSHOT
			DFS_Visit(v,h);
		}
		else
			if(vertice[v]->color == Grigio)
				M_ADJ[u][v]->setBack();
		else
			if(vertice[u]->Dtime < vertice[v]->Dtime)
				M_ADJ[u][v]->setForward();
			else  M_ADJ[u][v]->setCross();
//	SNAPSHOT

	}


vertice[u]->color = Nero;
vertice[u]->Ftime = time++; 



}



void Grafo::Initialize_Single_Source(int s)
{
	for (int i=0;i<NumeroDiVertici;i++)
	{
		vertice[i]->d = UNUSED_WEIGHT;
		vertice[i]->predecessore= NIL;
	}
  vertice[s]->d =0; 
  
}

void Grafo::Relax(int u,int v,int w)
{
	if(vertice[v]->d>vertice[u]->d+w)
	{
		vertice[v]->d = vertice[u]->d + w;
	    vertice[v]->predecessore = vertice[u]->idx;
		}
}
bool	 Grafo::Bellmann_Ford(/*Vertice sorgente, HANDLE hInstanceSockman*/)
{
Initialize_Single_Source(0);
int v,i,j;
	for( v=1;v< NumeroDiVertici;v++)	// |V|-1 volte
		for( i=0;i< NumeroDiVertici;i++)	
			for( j=0;j< NumeroDiVertici;j++)	
				if(M_ADJ[i][j]!=NULL)
				Relax(i,j,M_ADJ[i][j]->getPeso());

for( i=0;i< NumeroDiVertici;i++)						// etichetto i tree edges
	if(vertice[i]->predecessore!=NIL)
		M_ADJ[vertice[i]->predecessore][i]->setTree();					
		
		for( i=0;i< NumeroDiVertici;i++)	
			for( j=0;j< NumeroDiVertici;j++)	
				if(M_ADJ[i][j]!=NULL)
					if( vertice[j]->d > (vertice[i]->d+M_ADJ[i][j]->getPeso()) )
					{
						M_ADJ[i][j]->setBack();
						 
						char sztempBuffer[MAX_PRINT_BUFFER+1];					// Buffer for text to paint
					
						sprintf(sztempBuffer,"Ciclo negativo !\n %d > %d + %d = %d", vertice[j]->d,vertice[i]->d,M_ADJ[i][j]->getPeso(),vertice[i]->d+M_ADJ[i][j]->getPeso() );
						MessageBox(winGraphHandle, sztempBuffer,
								"" , MB_ICONINFORMATION    | MB_OK);
						
						return 	false; 
					}
return true;	
}



bool Grafo::color_legal(int i) //  controlla che gli adiacenti e gli incidenti del vertice i abbiano colore diverso da i
{
for(int j=0;j<= i;j++) 
	{
	if (i==j) continue; // ogni vertice '; adiacente a se stessp
	if( (is_adjacent(i,j)!=NULL)	// se c' un arco i->j
	&&  (vertice[i]->color == vertice[j]->color) )
		return false;
	if( (is_adjacent(j,i)!=NULL)		// se c' un arco j->i
	&&  (vertice[j]->color == vertice[i]->color) )
		return false;
	}
return true;	
}

void Grafo::init_Colorazione()
{
	for(int j=0;j<size_V();j++)
		vertice[j]->color=(enum Colore)-1;
}

bool Grafo::Colorazione(int i,int k)
{
	for(int j=0;j<k;j++)
	{
		vertice[i]->color=(enum Colore)j;
			if(color_legal(i))
			{
				if( (i+1) == size_V())
					return true;
				if (Colorazione(i+1,k))
				  return true;
			}
	}
return false;
}




void Grafo::init_Cricca()
{
	for(int j=0;j<size_V();j++)
		vertice[j]->color=Bianco;
}


bool Grafo::cricca_legal(int i) 
{
if(vertice[i]->color==Nero)
  	for (int j=0;j<i;j++)
		if((vertice[j]->color==Nero) && (is_adjacent(j,i)==NULL))
			return false;
		
return true;	
}


bool Grafo::enCricca(int i,int h, int k)
{
if(i<size_V())
{
	vertice[i]->color=Bianco;	// Bianco indica non appartenenza alla Cricca
	if (enCricca(i+1,h,k))
		return true;

	vertice[i]->color=Nero;		// Nero indica appartenenza alla Cricca
	if(cricca_legal(i))
	{
		if( (h+1) == k)
			return true;
		if (enCricca(i+1,h+1,k))
		  return true;
	}
vertice[i]->color=Bianco;	// senza questa riga l'algoritmo individua correttamente la presenza di cricche
}							// di dimensione h ma la colorazione della cricca pu risutare scorretta

return false;

}










