// Grafo.h: interface for the Grafo class.
//
//////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <math.h>



#if !defined(AFX_GRAFO_H__1984538B_DB1F_4069_997A_19F6A1EABF54__INCLUDED_)
#define AFX_GRAFO_H__1984538B_DB1F_4069_997A_19F6A1EABF54__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "CalliGraph.h"	// Prototypes and constants

enum TipoDiGrafo	{ NonOrientatoNonPesato = IDM_NON_ORIENTATO_NON_PESATO, OrientatoNonPesato,NonOrientatoPesato,OrientatoPesato };
enum Colore			{ Bianco = 0,Grigio,Nero,Rosso,Verde,Blu,Azzurro,Giallo,
					  Fucsia,Arancione,Verde_scuro};
enum EdgeType		{ Tree = 0 ,Back ,Cross ,Forward,Undef};


#define UNUSED_WEIGHT (32767)
#define NIL -1

#define MAX_VERTICI		'Z'-'A'+1


class Vertice	
{ 
private:


public:
	POINTS		pos;  	
	COLORREF	colore;
	char		*nome;
	int			idx;
	Colore		color;
	int			Dtime,Ftime;
	int			predecessore;
	int			spessore;
	int			d;	// distanza dal sorgente

	Vertice():color(Bianco),Dtime(-1),Ftime(-1){};
	Vertice(POINTS pos ,COLORREF col= 0); 
	
	Vertice &operator=(Vertice &rvalue);

	~Vertice(){ delete [] nome; };
	bool paint(HDC hdc);
	POINTS SetPos(POINTS coord){pos = coord;return pos;};
	POINTS SetPos(int x,int y){pos.x = x;pos.y = y;return pos;};


};
			


class Arco	
{ 
private:
	Vertice	*from,*to; 	
	COLORREF	colore;
	int			spessore;
	class		Grafo *G;
	int			peso;	
	EdgeType    tipo;
	
	char nome[MAX_PRINT_BUFFER+1];	 						

public:


	Arco(Vertice *from, Vertice *to, int peso,class Grafo *G)
	{
	Arco::from=from; 
	Arco::to=to;
	Arco::peso=peso;
	Arco::G=G;
	colore = RGB(0,0,0);
	spessore = SPESSORE_ARCO;
	tipo = Undef;
	};

	~Arco();
	void setUndef()	{colore = RGB(0,0,0);	  spessore = 2;tipo = Undef;	};	// 
	void setTree()	{colore = RGB(70,165,74); spessore = 4;tipo = Tree;		};	// 
	void setBack()	{colore = RGB(15,71,219); spessore = 2;tipo = Back;		};	//
	void setCross()	{colore = RGB(210,33,23); spessore = 2;tipo = Cross;	};	//
	void setForward(){colore= RGB(182,52,172);spessore = 2;tipo = Forward;	};	//
	int	 getPeso(){return peso;};

	bool paintDirect(HDC hdc);
	bool paintUndirect(HDC hdc);
	class Grafo * getGrafo(void){return G;};

}; 



class Grafo  
{
private:

	TipoDiGrafo TipoGrafo;    
	int			NumeroDiArchi;
	int			NumeroDiVertici;
	Arco		*M_ADJ[MAX_VERTICI][MAX_VERTICI];			// Matrice di Adiacenza
	Vertice		*vertice[MAX_VERTICI];
	char		nome[MAX_PRINT_BUFFER+1];				
	int			time;										// 
	
public:	
	Grafo(TipoDiGrafo tipo = OrientatoPesato, char *label="Grafo");	
	Grafo(Grafo& unGrafo);							//  nel caso "Grafo x = y" evita memberwise init
	
	~Grafo();
	

	Grafo&		operator=(Grafo& unGrafo); //overload di assegnamento memberwise copy
	
	
bool	add_vertex(Vertice* v);				/*ggiunge un Vertice*/


bool	add_edge(Arco &Edge );				/* Aggiunge un arco da Vertice1 a Vertice2 di peso weight nel grafo G. Se Vertice1 o Vertice2  non sono	vertici del grafo e se weight non e' compreso fra 0 e UNUSEDWEIGHT ritorna ERROR*/
bool	add_edge(Vertice &V1,Vertice &V2);	/* Aggiunge un arco da Vertice1 a Vertice2 di peso weight nel grafo G. Se Vertice1 o Vertice2  non sono	vertici del grafo e se weight non e' compreso fra 0 e UNUSEDWEIGHT ritorna ERROR*/
bool	add_edge(int V1,int V2, int peso );	/* Aggiunge un arco da Vertice1 a Vertice2 di peso weight nel grafo G. Se Vertice1 o Vertice2  non sono	vertici del grafo e se weight non e' compreso fra 0 e UNUSEDWEIGHT ritorna ERROR*/

bool	delete_edge(Arco &Edge); 			/*	Cancella l'arco da Vertice1 a Vertice2 in G. Se Vertice1 o Vertice2 non sono vertici del grafo, o se non c'e' un arco fra loro ritorna ERROR. */
bool	delete_edge(Vertice &V1,Vertice &V2); 			/*	Cancella l'arco da Vertice1 a Vertice2 in G. Se Vertice1 o Vertice2 non sono vertici del grafo, o se non c'e' un arco fra loro ritorna ERROR. */
bool	delete_edge(int V1, int V2); 			/*	Cancella l'arco da Vertice1 a Vertice2 in G. Se Vertice1 o Vertice2 non sono vertici del grafo, o se non c'e' un arco fra loro ritorna ERROR. */


Arco	*is_adjacent(int V1, int V2); 								/* Restituisce true se c'e un arco tra Vertice1 e Vertice2 in G, false altrimenti.*/ 

int		size_V(void){return 	NumeroDiVertici;};					/* Restituisce il numero di Vertici del grafo */
int		size_E(void){return 	NumeroDiArchi;  };					/* Restituisce il numero di Archi   del grafo */
char *  name(void)	{return		nome;			};

int		getTipo(void){return 	(int)TipoGrafo;   };
bool	setTipo(int T);

void	set_Orientato(void)		{};// non ancora implementati
void	set_Non_Orientato(void)	{};
void	set_Pesato(void)		{};
void	set_NonPesato(void)		{};

bool	is_Orientato(void)		{return (getTipo() & IDM_ORIENTATO)==IDM_ORIENTATO;};
bool	is_Non_Orientato(void)	{return !is_Orientato();};
bool	is_Pesato(void)			{return (getTipo() & IDM_PESATO)==IDM_PESATO;};
bool	is_NonPesato(void)		{return !is_Pesato();};



//edge  *edge_iterator( Vertice Vertice_number,edge *p_last_return);/* Edge_iterator restituisce l'arco che segue l'arco puntato da p_last_return nella  lista di adiacenza del vertice Vertice del grafo G.	*/


bool	paint(HDC pDC);

int		SnapToVertex(POINTS punto,int raggio);   // controlla se il punto  a distanza raggio da un vertice
bool	SnapToGrid(int Grid = 50);   // Auto Format Graph

void	DFS( HANDLE hInstanceSockman);
void	DFS_Visit(int sorgente, HANDLE hInstanceSockman);
void	Initialize_Single_Source(int s);

bool	Bellmann_Ford(/*Vertice s, HANDLE hInstanceSockman*/);
void	Relax(int u,int v,int w);

bool	Colorazione(int i,int k);	// meglio creare una classe a parte che erediti Grafo..
void	init_Colorazione();
bool	color_legal(int i); 
bool	cricca_legal(int i); 
bool	enCricca(int i,int h,int k);		
void	init_Cricca();
};







#endif // !defined(AFX_GRAFO_H__1984538B_DB1F_4069_997A_19F6A1EABF54__INCLUDED_)



















